// ==UserScript==
// @name           Spip Slideshow
// @namespace      doudoulinux.org
// @description    Turn a Spip article into a presentation
// @include        http://www.doudoulinux.org/
// ==/UserScript==


var StepByStep = false;		// hide lists in new slides or not
var LinkURL = document.URL + "#presentation"	// the URL of links
var Cycling = null;
var TimerInterval = 6000;

// internationalization
// from: http://24ways.org/2007/javascript-internationalisation
var i18n = {
thousands_sep: ' ',
"End of presentation": "Fin de la présentation",
"Switch to slideshow": "Basculer en présentation",
"Changes sections into presentation slides": "Changes les sections en pages de présentation",
"Previous slide": "Transparent précédent",
"Terminate": "Arrêter",
"Next entry or slide": "Entrée ou transparent suivant",
"Page %d/%d": "Page %d/%d",
"You are ordering %s, at a total cost of <strong>%s</strong> gold pieces.": '',
"Thank you.": ''
};

function _(msg) {
	if ( typeof(i18n) != 'undefined' && i18n[msg] ) {
		return i18n[msg];
	}
	return msg;
}

function sprintf(string) {
	var stringList = string.split('%');
	var result = stringList[0];
	var pattern = /^([ds])(.*)$/;
	for (var i=1; i<stringList.length; i++) {
		parts = pattern.exec(stringList[i]);
		if ( !parts || arguments[i]==null ) {
			continue;
		}
		if ( parts[1] == 'd' ) {
			result += parseInt(arguments[i], 10);
		} else if ( parts[1] == 's' ) {
			result += arguments[i];
		}
		result += parts[2];
	}
	return result;
}

// the presentation class
var spip_presentation = {
	setup: function() {
		this.sections = [];				// the section bodies
		this.presentationTitle = '';	// the presentation title!
		this.presentationDate = '';		// … and date
		this.slidetitles = [];			// the main slide titles
		this.license = '';				// the license text
		this.pos = -1;					// the current section nbr
		
		// inits
		var body = document.getElementsByTagName('body')[0];
		this.orig_markup = body.innerHTML;
		var markup = body.innerHTML;
		
		// collect section bodies
		var start = 0;
		var end = markup.indexOf('<h3 class="spip">');
		while (end != -1) {
			start = end;
			end = markup.indexOf('<h3 class="spip">', start+16);
			if (end == -1) {
				end = markup.indexOf('</div>', start+16);
				this.sections.push(markup.substr(start, end - start));
				break;
			}
			this.sections.push(markup.substr(start, end - start));
		}
		
		// collect presentation title
		start = markup.indexOf('<div class="cartouche">');
		end = markup.indexOf('</div>', start) + 6;
		if (start != -1 && end != -1) {
			start = markup.indexOf('<h1 class="">', start+23);
			end = markup.indexOf('</h1>', start+13) + 5;
			this.presentationTitle = markup.substr(start, end - start);
		} else {
			this.presentationTitle = '';
		}
		
		// collect date
		start = markup.indexOf('<small>', start) + 7;
		end = markup.indexOf('</small>', start);
		this.presentationDate = markup.substr(start, end - start);
		
		// collect main TOC entries
		var tocDiv = document.getElementById('tablematiere');
		var entries = tocDiv.getElementsByTagName('li');
		for (var i = 0; i < entries.length; i++) {
			this.slidetitles.push(entries[i].innerHTML);
		}
		
		// collect main TOC entries
		this.license = document.getElementById('license');
		
		// add a "show presentation" link
		/*if (this.sections.length > 0) {
			var heading = document.getElementsByTagName('h1')[0];
			var link = heading.appendChild(document.createElement('a'));
			link.setAttribute('id', 'spip_presentation_show');
			link.setAttribute('title', _("Changes sections into presentation slides"));
			link.appendChild(document.createTextNode( _("Switch to slideshow") ));
			link.setAttribute('href', LinkURL);
			link.addEventListener('click', this.start, false);
		}*/
	},
	
	start: function() {
		// reset the page body
		var body = document.getElementsByTagName('body')[0];
		body.setAttribute('id', 'presentation');
		body.innerHTML = "";
		
		// add a title box
		var titlebox = body.appendChild(document.createElement('div'));
		titlebox.setAttribute('id', 'spip_presentation_title');
		titlebox.innerHTML = spip_presentation.presentationTitle;
		
		// add the date
		var datebox = titlebox.appendChild(document.createElement('p'));
		datebox.setAttribute('id', 'spip_presentation_date');
		datebox.innerHTML = spip_presentation.presentationDate;
		
		// add a slide navigation bar
		var navbox = titlebox.appendChild(document.createElement('div'));
		navbox.setAttribute('id', 'spip_presentation_nav');

		var prev = navbox.appendChild(document.createElement('a'));
		//prev.innerHTML = "←";
		prev.setAttribute('title', _("Previous slide"));
		prev.setAttribute('class', 'prev');
		prev.setAttribute('href', LinkURL);
		prev.addEventListener('click', function() {
			spip_presentation.prev();
		}, false);
		
		var stop = navbox.appendChild(document.createElement('a'));
		//stop.innerHTML = "↑";
		stop.setAttribute('title', _("Terminate"));
		stop.setAttribute('class', 'stop');
		stop.setAttribute('href', LinkURL);
		stop.addEventListener('click', function() {
			spip_presentation.stop();
		}, false);
		
		var next = navbox.appendChild(document.createElement('a'));
		//next.innerHTML = "→";
		next.setAttribute('title', _("Next entry or slide"));
		next.setAttribute('class', 'next');
		next.setAttribute('href', LinkURL);
		next.addEventListener('click', function() {
			spip_presentation.next();
		}, false);
		
		// add the logo
		var logo = body.appendChild(document.createElement('img'));
		logo.setAttribute('id', 'logo');
		logo.setAttribute('src', 'rmll-2013_fichiers/logo.png');
		logo.setAttribute('alt', 'logo');
		
		// add the cycling indicator
		var cycling = body.appendChild(document.createElement('div'));
		cycling.setAttribute('id', 'cycling');
		var cycleimg = cycling.appendChild(document.createElement('img'));
		cycleimg.setAttribute('src', 'rmll-2013_fichiers/throbber.gif');
		cycleimg.setAttribute('alt', 'cycling');
		
		// add the contents box
		var slide = body.appendChild(document.createElement('div'));
		slide.setAttribute('id', 'spip_presentation');
		
		// add the footer box
		body.appendChild(spip_presentation.license);
		var pagebox = body.appendChild(document.createElement('div'));
		pagebox.setAttribute('id', "spip_presentation_footer");
		pagebox.appendChild(document.createElement('span'));
		
		// display 1st transparent
		spip_presentation.setPage(StepByStep);
		document.addEventListener('keypress', spip_presentation.keypress, false);
	},
	
	setPage: function(hideLists) {
		// set transparent contents
		var slide = document.getElementById('spip_presentation');
		if (this.pos == spip_presentation.sections.length ) {
			// presentation end
			slide.innerHTML = '';
			var textbox = slide.appendChild(document.createElement('p'));
			textbox.setAttribute('id', 'presentation-end');
			textbox.appendChild(document.createTextNode(_("End of presentation")));

		} else if (this.pos == -1 ) {
			// presentation TOC
			slide.innerHTML = '';
			var h = slide.appendChild(document.createElement('h2'));
			h.setAttribute('id', 'presentation_title');
			var start = this.presentationTitle.indexOf('>') + 1;
			var end = this.presentationTitle.indexOf('</h1>', start);
			h.innerHTML = this.presentationTitle.substr(start, end - start);

			var list = slide.appendChild(document.createElement('ul'));
			list.setAttribute('id', 'presentation_toc');
			var item;
			for (var i = 0; i < this.slidetitles.length; i++) {
				item = list.appendChild(document.createElement('li'));
				item.innerHTML = this.slidetitles[i];
			}
		} else {
			// presentation slide
			slide.innerHTML = spip_presentation.sections[this.pos];
			
			// remove width and height in Html code
			var entries = slide.getElementsByTagName("img");
			for (var i = 0; i < entries.length; i++) {
				if ( entries[i].hasAttribute("width") ) {
					entries[i].removeAttribute("width");
				}
				if ( entries[i].hasAttribute("height") ) {
					entries[i].removeAttribute("height");
				}
				if ( entries[i].hasAttribute("style") ) {
					entries[i].removeAttribute("style");
				}
			}
			var entries = slide.getElementsByTagName("span");
			for ( i = 0; i < entries.length; i++) {
				if ( entries[i].hasAttribute("style") ) {
					entries[i].removeAttribute("style");
				}
			}
			
			// hide list items if any
			if (hideLists) {
				var items = slide.getElementsByTagName('li');
				for (var i = 1; i < items.length; i++) {
					items[i].style.display = 'none';
				}
			}
		}
		
		// display page number
		if (this.pos > -1 ) {
			var footbox = document.getElementById('spip_presentation_footer');
			var items = footbox.getElementsByTagName('span');
			items[0].innerHTML = sprintf( _("Page %d/%d"), this.pos+1, (this.sections.length+1) );
		}
	},
	
	stop: function() {
		var body = document.getElementsByTagName('body')[0];
		body.innerHTML = spip_presentation.orig_markup;
		body.setAttribute('id', '');
		spip_presentation.setup();
		document.removeEventListener('keypress', spip_presentation.keypress, false);
	},
	
	keypress: function(event) {
		if ( event.which == null ) {
			var key = event.keyCode;
		} else {
			var key = event.which;
		}
		//alert("key =" + key);
		
		// manage auto-cycling
		if ( Cycling ) {
			window.clearInterval(Cycling);
			if ( key == 43 ) {
				// faster
				TimerInterval = TimerInterval + 1000;
				Cycling = window.setInterval(spip_presentation.next, TimerInterval);
			} else if ( key == 45 ) {
				//slower
				if ( TimerInterval > 3000 ) {
					TimerInterval = TimerInterval - 1000;
				}
				Cycling = window.setInterval(spip_presentation.next, TimerInterval);
			} else {
				// finish
				document.getElementById("cycling").style.display = "none";
			}
		}
		
		// manual commands
		//NB: webkit doesn't capture arrow keys…
		if ( key == 39 || key == 32 || key == 110 ) {
			// spacebar (keyCode=32) is captured by Firefox "search on type" feature?
			spip_presentation.next();
		} else if ( key == 37 || key == 112 ) {
			spip_presentation.prev();
		} else if ( key == 27 ) {
			spip_presentation.stop();
		} else if ( key == 99 ) {
			Cycling = window.setInterval(spip_presentation.next, TimerInterval);
			document.getElementById("cycling").style.display = "block";
		}
	},
	
	prev: function() {
		this.pos--;
		if (this.pos == -2) {
			this.pos = this.sections.length;
		}
		spip_presentation.setPage(false);
	},
	
	next: function() {
		// if list elements, show them one after another before switching to next page
		var slide = document.getElementById('spip_presentation');
		var items = slide.getElementsByTagName('li');
		
		var hidden = 0;
		for (var i = 0; i < items.length; i++) {
			if (items[i].style.display == 'none') {
				hidden++;
			}
		}
		if (hidden > 0) {
			items[items.length - hidden].style.display = 'list-item';
			return;
		}
		
		// switch to next page
		// NB: we can't use "this" because the auto-cycling timer doesn't know it
		spip_presentation.pos++;
		if (spip_presentation.pos > spip_presentation.sections.length) {
			spip_presentation.pos = -1;
		}
		spip_presentation.setPage(StepByStep);
	}
};

//spip_presentation.setup();
